#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `highresnet` package."""


import unittest
from click.testing import CliRunner

from highresnet.cli import deepgif


class TestHighresnet(unittest.TestCase):
    """Tests for `highresnet` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_000_something(self):
        """Test something."""

    def test_command_line_interface(self):
        """Test the CLI."""
